import { ChatInputCommandInteraction, PermissionFlagsBits, SlashCommandBuilder } from "discord.js";
import { Badword, database } from "../data";
import { getGuildSetting, isPremiumActive } from "../tools/data";
import getDefaultEmbed from "../tools/defaultEmbeds";
import moment from "moment";
import { Color, Emoji } from "../tools/design";
import { getGuildChannel, getChannelPermission } from "../tools/discord";

const builder = new SlashCommandBuilder();
builder.setName("showsettings");
builder.setDescription("Show the current settings of this guild");
builder.setDMPermission(false);
builder.setDefaultMemberPermissions(PermissionFlagsBits.ManageGuild);

async function execute(interaction: ChatInputCommandInteraction): Promise<void> {
    if (!interaction.inGuild()) throw new Error("Interaction was performed outside guild context");

    const settings = await getGuildSetting(interaction.guildId);
    const isPremium = await isPremiumActive(settings.isPremiumUntil);
    const wordCount = await database.getRepository(Badword).count({
        where: {
            guildID: interaction.guildId
        }
    });

    const logChannel = settings.notificationChannelID ? await getGuildChannel(interaction.guildId, settings.notificationChannelID) : null;

    const embed = getDefaultEmbed();
    embed.setTitle(`Settings from guild ${interaction.guild?.name || ""} (${interaction.guildId})`);
    embed.setDescription(isPremium ? `${Emoji.PREMIUM} your subscription ends in ${moment(settings.isPremiumUntil).fromNow(true)}` : `Consider Premium status to get an increased blocklist`);
    embed.setColor(isPremium ? Color.PREMIUM_ORANGE : Color.INFORMING_BLUE);
    embed.addFields({
        name: "Premium",
        value: isPremium ? `${Emoji.PREMIUM} active` : `${Emoji.SWITCH_OFF} inactive`,
        inline: true
    }, {
        name: "Logchannel",
        value: settings.notificationChannelID ? `<#${settings.notificationChannelID}>` : "Not configured",
        inline: true
    }, {
        name: "Words in Blocklist",
        value: `${wordCount}/${isPremium ? "100" : "10"}`,
        inline: true
    }, {
        name: `Preserve data on server leave is ${settings.preserveDataOnGuildLeave ? "active" : "inactive"}`,
        value: settings.preserveDataOnGuildLeave ? "Your settings will be saved even if the bot gets kicked" : "Your settings will be deleted as soon as the bot leaves this server"
    }, {
        name: `${Emoji.WAVING} Found a bug? Want to request a feature? Say hello?`,
        value: "Join the support server at https://go.astrogd.eu/discord"
    });

    const embeds = [];
    embeds.push(embed);

    const invalidLogChannelEmbed = getDefaultEmbed();
    invalidLogChannelEmbed.setColor(Color.WARNING_YELLOW);
    invalidLogChannelEmbed.setTitle(`${Emoji.SECURITY_CHALLENGE_FAILED} Logchannel is misconfigured!`);
    invalidLogChannelEmbed.setDescription("The bot is unable to read and/or write in the configured logChannel. Please adjust your permissions.");

    if (logChannel && logChannel.isTextBased()) {
        const permissions = await getChannelPermission(logChannel);
        if (!permissions || !permissions.has(PermissionFlagsBits.ViewChannel) || !permissions.has(PermissionFlagsBits.SendMessages)) {
            embeds.push(invalidLogChannelEmbed);
        }
    }

    if ((!logChannel || !logChannel.isTextBased()) && settings.notificationChannelID) {
        embeds.push(invalidLogChannelEmbed);
    }

    interaction.reply({
        embeds: embeds,
        ephemeral: true
    }).catch();
}

export {
    builder,
    execute
}